package Bank;
/**
 * simple representation of bank accounts
 * 
 * @author Tim Wahls
 * 
 */
public class Account
{
    private int acctNumber; // account number
    private int balance; // in pennies
    private int interestRate; // in pct.
    
    /**
     * Constructor for creating an Account object with
     * default values for the fields.
     */
    public Account() {
        balance = 0;
        acctNumber = 0;
        interestRate = 2;
    }
    
    /**
     * Constructor for objects of class Account
     * @param initAcctNumber the new account number
     */
    public Account(int initAcctNumber)
    {
        // initialise instance variables
        acctNumber = initAcctNumber;
        balance = 0;
        interestRate = 2;
    }
    
    /**
     * Constructor for an Account with a specified
     * account number and interest rate.
     * @param initAcctNumber the account number.
     * @param initInterestRate the interest rate.
     */
    public Account(int initAcctNumber, int initInterestRate) {
        acctNumber = initAcctNumber;
        balance = 0;
        interestRate = initInterestRate;
    }
    
    /**
     * set the account number
     * @param acct the new account number
     */
    public void setAccountNumber(int acct)
    {
        acctNumber = acct;
    }
    
    /**
     * get the account number
     * @return the account number
     */
    public int getAccountNumber() {
        return acctNumber;
    }
    
    /**
     * Get the interest rate.
     * @return the interest rate.
     */
    public int getInterestRate() {
        return interestRate;
    }
    
    /**
     *  deposit money into the account
     *  @param depAmount the amount to deposit
     */
    public void deposit(int depAmount) {
        if (depAmount > 0) {
           balance = balance + depAmount;
        } else {
           System.out.println("Error: invalid deposit amount!");
        }
    }
    
    /** 
     * withdraw money from the account
     * @param withAmount the amount to withdraw
     */
    public void withdraw(int withAmount) {
        if (balance >= withAmount) {
           balance = balance - withAmount;
        } else {
           System.out.println("Error: there are only " + balance +
                              " cents in this account!");
        }
    } 
    
    /**
     * Add interest to the account's balance based upon
     * its interest rate.
     */
    public void addInterest() {
        int interest = (balance * interestRate) / 100;
        balance = balance + interest;
    }

    /** 
     * check the account balance (an accessor method)
     * @return the account balance
     */
    public int getBalance() {
        return balance;
    }
    
    /**
     * display the account number and balance on the screen
     */
    public void print() {
      int dollars = balance / 100;
      int dimes = (balance % 100) / 10;
      int pennies = (balance % 100) % 10;
      System.out.println("Account number: " + acctNumber);
      System.out.println("Balance is: $" + dollars + "." + 
                         dimes + "" + pennies);
    }
    
}
